@@ -156,7 +156,7 @@ void CreateInterfaceTool::estimateSize()
 	int input_width = 0;
 	int output_width = 0;
 
-	for(pi = minfo.ports.begin(); pi != minfo.ports.end(); pi++)
+	for(pi = minfo.ports.begin(); pi != minfo.ports.end(); ++pi)
 	{
 		if(pi->direction == Arts::input)
 			input_width++;
@@ -179,7 +179,7 @@ void CreateInterfaceTool::performCreate(int x, int y)
 	int output_width = 0;
 
 	for(pi = minfo.ports.begin(), pni = minfo.portnames.begin();
-		pi != minfo.ports.end(); pi++, pni++)
+		pi != minfo.ports.end(); ++pi, pni++)
 	{
 		StructurePort *port = 0;
 		/*
@@ -209,7 +209,7 @@ CreateInterfaceTool::CreateInterfaceTool(ModuleWidget *mwidget,
 	this->minfo = minfo;
 
 	vector<Arts::PortType>::iterator pi;
-	for(pi = this->minfo.ports.begin(); pi != this->minfo.ports.end(); pi++)
+	for(pi = this->minfo.ports.begin(); pi != this->minfo.ports.end(); ++pi)
 	{
 		/* reverse since we're inside the interface, not outside */
 		if(pi->direction == Arts::input)
@@ -262,7 +262,7 @@ void MoveComponentsTool::mouseMoveEvent(QMouseEvent *e)
 	std::list<StructureComponent *>::iterator i;
 	std::list<StructureComponent *> *ComponentList = mwidget->structure->getComponentList();
 
-	for(i = ComponentList->begin();i != ComponentList->end();i++)
+	for(i = ComponentList->begin();i != ComponentList->end();++i)
 	{
 		StructureComponent *c = *i;
 		if(c->selected() && !mwidget->hasSpace(c, c->x() + dx, c->y() + dy, true))
@@ -270,7 +270,7 @@ void MoveComponentsTool::mouseMoveEvent(QMouseEvent *e)
 	}
 
 	mwidget->beginUpdate();
-	for(i = ComponentList->begin();i != ComponentList->end();i++)
+	for(i = ComponentList->begin();i != ComponentList->end();++i)
 	{
 		StructureComponent *c = *i;
 		if(c->selected())
