@@ -316,7 +316,7 @@ Playlist::Playlist(PlaylistCollection *collection, const QString &name,
     m_allowDuplicates(false),
     m_polished(false),
     m_applySharedSettings(true),
-    m_mousePressed(false),
+    m_columnWidthModeChanged(false),
     m_disableColumnWidthUpdates(true),
     m_time(0),
     m_widthsDirty(true),
@@ -339,7 +339,7 @@ Playlist::Playlist(PlaylistCollection *collection, const PlaylistItemList &items
     m_allowDuplicates(false),
     m_polished(false),
     m_applySharedSettings(true),
-    m_mousePressed(false),
+    m_columnWidthModeChanged(false),
     m_disableColumnWidthUpdates(true),
     m_time(0),
     m_widthsDirty(true),
@@ -363,7 +363,7 @@ Playlist::Playlist(PlaylistCollection *collection, const QFileInfo &playlistFile
     m_allowDuplicates(false),
     m_polished(false),
     m_applySharedSettings(true),
-    m_mousePressed(false),
+    m_columnWidthModeChanged(false),
     m_disableColumnWidthUpdates(true),
     m_time(0),
     m_widthsDirty(true),
@@ -386,7 +386,7 @@ Playlist::Playlist(PlaylistCollection *collection, bool delaySetup) :
     m_allowDuplicates(false),
     m_polished(false),
     m_applySharedSettings(true),
-    m_mousePressed(false),
+    m_columnWidthModeChanged(false),
     m_disableColumnWidthUpdates(true),
     m_time(0),
     m_widthsDirty(true),
@@ -438,7 +438,7 @@ int Playlist::time() const
     // Since this method gets a lot of traffic, let's optimize for such.
 
     if(!m_addTime.isEmpty()) {
-	for(QValueList<PlaylistItem::Pointer>::ConstIterator it = m_addTime.begin();
+	for(PlaylistItemList::ConstIterator it = m_addTime.begin();
 	    it != m_addTime.end(); ++it)
 	{
 	    if(*it)
@@ -449,7 +449,7 @@ int Playlist::time() const
     }
 
     if(!m_subtractTime.isEmpty()) {
-	for(QValueList<PlaylistItem::Pointer>::ConstIterator it = m_subtractTime.begin();
+	for(PlaylistItemList::ConstIterator it = m_subtractTime.begin();
 	    it != m_subtractTime.end(); ++it)
 	{
 	    if(*it)
@@ -574,6 +574,8 @@ void Playlist::clearItem(PlaylistItem *item, bool emitChanged)
     m_search.clearItem(item);
 
     m_history.remove(item);
+    m_addTime.remove(item);
+    m_subtractTime.remove(item);
 
     delete item;
     if(emitChanged)
@@ -929,6 +931,7 @@ void Playlist::slotShowPlaying()
     m_collection->raise(l);
 
     l->setSelected(playingItem(), true);
+    l->setCurrentItem(playingItem());
     l->ensureItemVisible(playingItem());
 }
 
@@ -1097,24 +1100,32 @@ bool Playlist::eventFilter(QObject *watched, QEvent *e)
 {
     if(watched == header()) {
 	switch(e->type()) {
+	case QEvent::MouseMove:
+	{
+	    if((static_cast<QMouseEvent *>(e)->state() & LeftButton) == LeftButton &&
+		!action<KToggleAction>("resizeColumnsManually")->isChecked())
+	    {
+		m_columnWidthModeChanged = true;
+
+		action<KToggleAction>("resizeColumnsManually")->setChecked(true);
+		slotColumnResizeModeChanged();
+	    }
+
+	    break;
+	}
 	case QEvent::MouseButtonPress:
 	{
-	    switch(static_cast<QMouseEvent *>(e)->button()) {
-	    case RightButton:
+	    if(static_cast<QMouseEvent *>(e)->button() == RightButton)
 		m_headerMenu->popup(QCursor::pos());
-		break;
-	    case LeftButton:
-		m_mousePressed = true;
-		break;
-	    default:
-		break;
-	    }
+
 	    break;
 	}
 	case QEvent::MouseButtonRelease:
 	{
-	    if(static_cast<QMouseEvent *>(e)->button() == LeftButton)
-		m_mousePressed = false;
+	    if(m_columnWidthModeChanged) {
+		m_columnWidthModeChanged = false;
+		notifyUserColumnWidthModeChanged();
+	    }
 
 	    if(!manualResize() && m_widthsDirty)
 		QTimer::singleShot(0, this, SLOT(slotUpdateColumnWidths()));
@@ -1291,12 +1302,19 @@ void Playlist::viewportResizeEvent(QResizeEvent *re)
 
 void Playlist::insertItem(QListViewItem *item)
 {
+    // Because we're called from the PlaylistItem ctor, item may not be a
+    // PlaylistItem yet (it would be QListViewItem when being inserted.  But,
+    // it will be a PlaylistItem by the time it matters, but be careful if
+    // you need to use the PlaylistItem from here.
+
     m_addTime.append(static_cast<PlaylistItem *>(item));
     KListView::insertItem(item);
 }
 
 void Playlist::takeItem(QListViewItem *item)
 {
+    // See the warning in Playlist::insertItem.
+
     m_subtractTime.append(static_cast<PlaylistItem *>(item));
     KListView::takeItem(item);
 }
@@ -2261,22 +2279,20 @@ void Playlist::slotCreateGroup()
 	new Playlist(m_collection, selectedItems(), name);
 }
 
+void Playlist::notifyUserColumnWidthModeChanged()
+{
+    KMessageBox::information(this,
+			     i18n("Manual column widths have been enabled.  You can "
+				  "switch back to automatic column sizes in the view "
+				  "menu."),
+			     i18n("Manual Column Widths Enabled"),
+			     "ShowManualColumnWidthInformation");
+}
+
 void Playlist::slotColumnSizeChanged(int column, int, int newSize)
 {
     m_widthsDirty = true;
     m_columnFixedWidths[column] = newSize;
-
-    if(m_mousePressed && !action<KToggleAction>("resizeColumnsManually")->isChecked()) {
-	KMessageBox::information(this,
-				 i18n("Manual column widths have been enabled.  You can "
-				      "switch back to automatic column sizes in the view "
-				      "menu."),
-				 i18n("Manual Column Widths Enabled"),
-				 "ShowManualColumnWidthInformation");
-	
-	action<KToggleAction>("resizeColumnsManually")->setChecked(true);
-	slotColumnResizeModeChanged();
-    }
 }
 
 void Playlist::slotInlineCompletionModeChanged(KGlobalSettings::Completion mode)
