@@ -29,7 +29,7 @@ extern "C"
 	#include <cdda_interface.h>
 	#include <cdda_paranoia.h>
 	void paranoiaCallback(long, int);
-	
+
 	#include <kdemacros.h>
 	KDE_EXPORT int kdemain(int argc, char ** argv);
 }
@@ -104,14 +104,13 @@ enum Which_dir {
 
 class AudioCDProtocol::Private {
 public:
-	Private() {
+	Private() : cd(KCompactDisc::Asynchronous) {
 		clearURLargs();
 		s_info = i18n("Information");
 		s_fullCD = i18n("Full CD");
 	}
 
 	void clearURLargs() {
-		discid = 0;
 		req_allTracks = false;
 		which_dir = Unknown;
 		req_track = -1;
@@ -129,7 +128,7 @@ public:
 	QString device; // URL settable
 	int paranoiaLevel; // URL settable
 	bool reportErrors;
-	
+
 	// Directory strings, never change after init
 	QString s_info;
 	QString s_fullCD;
@@ -138,6 +137,7 @@ public:
 	unsigned discid;
 	unsigned tracks;
 	bool trackIsAudio[100];
+	KCompactDisc cd; // keep it around so that we don't assume the disk changed between every stat()
 
 	// CDDB items
 	KCDDB::CDDB::Result cddbResult;
@@ -164,7 +164,7 @@ AudioCDProtocol::AudioCDProtocol(const QCString & protocol, const QCString & poo
 	d = new Private;
 
 	// Add encoders
-	AudioCDEncoder::find_all_plugins(this, encoders);
+	AudioCDEncoder::findAllPlugins(this, encoders);
 	encoderTypeCDA = encoderFromExtension(".cda");
 	encoderTypeWAV = encoderFromExtension(".wav");
 	encoders.setAutoDelete(true);
@@ -213,28 +213,66 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 		return 0;
 
 	// Update our knowledge of the disc
-	KCompactDisc cd(KCompactDisc::Asynchronous);
-	// TODO which one is right?
-	// qDebug("\"%s\" \"%s\"", drive->cdda_device_name, drive->ioctl_device_name);
-#if defined(Q_OS_LINUX)
-	cd.setDevice(drive->cdda_device_name, 50, false);
-#elif defined(Q_OS_FREEBSD)
-	cd.setDevice(drive->dev->device_path);
+
+#if defined(__linux__)
+       if(drive->ioctl_device_name && drive->ioctl_device_name[0])
+         d->cd.setDevice(drive->ioctl_device_name, 50, false);
+       else
+         d->cd.setDevice(drive->cdda_device_name, 50, false);
+#else
+        d->cd.setDevice(drive->cdda_device_name, 50, false);
 #endif
-	
-	if (cd.discId() != d->discid && cd.discId() != cd.missingDisc){
-		d->discid = cd.discId();
-		d->tracks = cd.tracks();
-		for(uint i=0; i< cd.tracks(); i++)
-			d->trackIsAudio[i] = cd.isAudio(i+1);
+
+#if 0
+	// FreeBSD's cdparanoia as of january 5th 2006 has rather broken
+	// support for non-SCSI devices. Although it finds ATA cdroms just
+	// fine, there is no straightforward way to discover the device
+	// name associated with the device, which throws the rest of audiocd
+	// for a loop.
+	//
+	if ( !(drive->dev) || (COOKED_IOCTL == drive->interface) )
+	{
+		// For ATAPI devices, we have no real choice. Use the
+		// user selected value, even if there is none.
+		//
+		kdWarning(7117) << "Found an ATAPI device, assuming it is the one specified by the user." << endl;
+		d->cd.setDevice( d->device );
+	}
+	else
+	{
+		kdDebug(7117) << "Found a SCSI or ATAPICAM device." << endl;
+		if ( strlen(drive->dev->device_path) > 0 )
+		{
+			d->cd.setDevice( drive->dev->device_path );
+		}
+		else
+		{
+			// But the device_path can be empty under some
+			// circumstances, so build a representation from
+			// the unit number and SCSI device name.
+			//
+			QString devname = QString::fromLatin1( "/dev/%1%2" )
+				.arg( drive->dev->given_dev_name )
+				.arg( drive->dev->given_unit_number ) ;
+			kdDebug(7117) << "  Using derived name " << devname << endl;
+			d->cd.setDevice( devname );
+		}
+	}
+#endif
+
+	if (d->cd.discId() != d->discid && d->cd.discId() != d->cd.missingDisc){
+		d->discid = d->cd.discId();
+		d->tracks = d->cd.tracks();
+		for(uint i=0; i< d->cd.tracks(); i++)
+			d->trackIsAudio[i] = d->cd.isAudio(i+1);
 
 		KCDDB::Client c;
-		d->cddbResult = c.lookup(cd.discSignature());
+		d->cddbResult = c.lookup(d->cd.discSignature());
 		d->cddbList = c.lookupResponse();
 		d->cddbBestChoice = c.bestLookupResponse();
 		generateTemplateTitles();
 	}
-	
+
 	// Determine what file or folder that is wanted.
 	d->fname = url.fileName(false);
 	QString dname = url.directory(true, false);
@@ -410,10 +448,9 @@ void AudioCDProtocol::get(const KURL & url)
 	KCDDB::CDInfo info;
 	if(d->cddbResult == KCDDB::CDDB::Success){
 		info = d->cddbBestChoice;
-	
+
 		int track = d->req_track;
-		
-		QString trackName;
+
 		// hack
 		// do we rip the whole CD?
 		if (d->req_allTracks){
@@ -423,9 +460,6 @@ void AudioCDProtocol::get(const KURL & url)
 		}
 		encoder->fillSongInfo(info, track, "");
 	}
-	else {
-		encoder->fillSongInfo(info, d->req_track, QString("CDDBDISCID=%1").arg(d->cddbBestChoice.id));
-	}
 	long totalByteCount = CD_FRAMESIZE_RAW * (lastSector - firstSector + 1);
 	long time_secs = (8 * totalByteCount) / (44100 * 2 * 16);
 
@@ -723,7 +757,7 @@ struct cdrom_drive *AudioCDProtocol::getDrive()
 			error(KIO::ERR_SLAVE_DEFINED, i18n("Device doesn't have write permissions for this account.  Check the write permissions on the device."));
 		else if(!fi.exists())
 			error(KIO::ERR_DOES_NOT_EXIST, d->device);
-		else error(KIO::ERR_SLAVE_DEFINED, 
+		else error(KIO::ERR_SLAVE_DEFINED,
 i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ as yourself (not root). Do you see a track list? If not, make sure you have permission to access the CD device. If you are using SCSI emulation (possible if you have an IDE CD writer) then make sure you check that you have read and write permissions on the generic SCSI device, which is probably /dev/sg0, /dev/sg1, etc.. If it still does not work, try typing audiocd:/?device=/dev/sg0 (or similar) to tell kio_audiocd which device your CD-ROM is."));
 		return 0;
 	}
@@ -732,10 +766,10 @@ i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ
 	{
 		kdDebug(7117) << "cdda_open failed" << endl;
 		error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
-		cdda_close(drive); 
+		cdda_close(drive);
 		return 0;
 	}
-	
+
 	return drive;
 }
 
@@ -814,7 +848,11 @@ void AudioCDProtocol::paranoiaRead(
 		if(encoderProcessed == -1){
 			kdDebug(7117) << "Encoder processing error, stopping." << endl;
 			ok = false;
-			error( ERR_SLAVE_DEFINED, i18n( "Couldn't read %1: encoding failed" ).arg( fileName ) );
+			QString errMsg = i18n( "Couldn't read %1: encoding failed" ).arg( fileName );
+			QString details = encoder->lastErrorMessage();
+			if ( !details.isEmpty() )
+			    errMsg += "\n" + details;
+			error( ERR_SLAVE_DEFINED, errMsg );
 			break;
 		}
 		processed += encoderProcessed;
@@ -873,7 +911,7 @@ void AudioCDProtocol::paranoiaRead(
 		/**
 		 * End estimation.
 		 */
-		//qDebug("processed: %d, totalSize: %d", processed, estSize);
+		//qDebug("processed: %ld, totalSize: %ld", processed, estSize);
 		processedSize(processed);
 	}
 
@@ -946,7 +984,7 @@ void AudioCDProtocol::parseURLArgs(const KURL & url)
  */
 void AudioCDProtocol::loadSettings()
 {
-	KConfig *config = new KConfig(QFL1("kcmaudiocdrc"));
+	KConfig *config = new KConfig(QFL1("kcmaudiocdrc"), true /*readonly*/, false /*no kdeglobals*/);
 
 	config->setGroup(QFL1("CDDA"));
 
@@ -979,12 +1017,32 @@ void AudioCDProtocol::loadSettings()
 	d->albumTemplate = config->readEntry("album_template", "%{albumartist} - %{albumtitle}");
 	d->rsearch = config->readEntry("regexp_search");
 	d->rreplace = config->readEntry("regexp_replace");
+	// if the regular expressions are enclosed in qoutes. remove them
+	// otherwise it is not possible to search for a space " ", since an empty (only spaces) value is not 
+	// supported by KConfig, so the space has to be qouted, but then here the regexp searches really for " "
+	// instead of just the space. Alex
+	QRegExp qoutedString("^\".*\"$");
+	if (qoutedString.exactMatch(d->rsearch))
+	{
+        	d->rsearch=d->rsearch.mid(1, d->rsearch.length()-2);
+	}
+	if (qoutedString.exactMatch(d->rreplace))
+	{
+		d->rreplace=d->rreplace.mid(1, d->rreplace.length()-2);
+	}
 
 	// Tell the encoders to load their settings
-	AudioCDEncoder *encoder;
-	for ( encoder = encoders.first(); encoder; encoder = encoders.next() ){
-		encoder->init();
-		encoder->loadSettings();
+	AudioCDEncoder *encoder = encoders.first();
+	while ( encoder ) {
+		if ( encoder->init() ) {
+			kdDebug(7117) << "Encoder for " << encoder->type() << " is available." << endl;
+			encoder->loadSettings();
+			encoder = encoders.next();
+		} else {
+			kdDebug(7117) << "Encoder for " << encoder->type() << " is NOT available." << endl;
+			encoders.remove( encoder );
+			encoder = encoders.current();
+		}
 	}
 
 	delete config;
