@@ -26,12 +26,13 @@
 #include <kprocess.h>
 #include <kdebug.h>
 
-#include <kglobal.h>  
+#include <kglobal.h>
 #include <klocale.h>
 #include <kapplication.h>
 #include <qfileinfo.h>
 #include <ktempfile.h>
 #include <kstandarddirs.h>
+#include "collectingprocess.h"
 
 extern "C"
 {
@@ -47,6 +48,9 @@ class EncoderLame::Private
 public:
 	int bitrate;
 	bool waitingForWrite;
+	bool processHasExited;
+	QString lastErrorMessage;
+	QStringList genreList;
 	uint lastSize;
 	KProcess *currentEncodeProcess;
 	KTempFile *tempFile;
@@ -55,6 +59,7 @@ public:
 EncoderLame::EncoderLame(KIO::SlaveBase *slave) : QObject(), AudioCDEncoder(slave) {
 	d = new Private();
 	d->waitingForWrite = false;
+	d->processHasExited = false;
 	d->lastSize = 0;
 	loadSettings();
 }
@@ -64,27 +69,51 @@ EncoderLame::~EncoderLame(){
 }
 
 QWidget* EncoderLame::getConfigureWidget(KConfigSkeleton** manager) const {
-  (*manager) = Settings::self();
-  KGlobal::locale()->insertCatalogue("audiocd_encoder_lame");
-  EncoderLameConfig *config = new EncoderLameConfig();
-  config->cbr_settings->hide();
-  return config;
+	(*manager) = Settings::self();
+	KGlobal::locale()->insertCatalogue("audiocd_encoder_lame");
+	EncoderLameConfig *config = new EncoderLameConfig();
+	config->cbr_settings->hide();
+	return config;
 }
 
 bool EncoderLame::init(){
 	// Determine if lame is installed on the system or not.
-	KProcess proc;
-	proc << "which" << "lame";
-	proc.start(KProcess::Block);
-	if(proc.exitStatus() == 0)
+	if ( KStandardDirs::findExe( "lame" ).isEmpty() )
 		return false;
+
+	// Ask lame for the list of genres it knows; otherwise it barfs when doing
+	// e.g. lame --tg 'Vocal Jazz'
+    CollectingProcess proc;
+	proc << "lame" << "--genre-list";
+	proc.start(KProcess::Block, KProcess::Stdout);
+
+	if(proc.exitStatus() != 0)
+		return false;
+
+	const QByteArray data = proc.collectedStdout();
+	QString str;
+	if ( !data.isEmpty() )
+		str = QString::fromLocal8Bit( data, data.size() );
+
+	d->genreList = QStringList::split( '\n', str );
+	// Remove the numbers in front of every genre
+	for( QStringList::Iterator it = d->genreList.begin(); it != d->genreList.end(); ++it ) {
+		QString& genre = *it;
+		uint i = 0;
+		while ( i < genre.length() && ( genre[i].isSpace() || genre[i].isDigit() ) )
+			++i;
+		genre = genre.mid( i );
+
+	}
+	//kdDebug(7117) << "Available genres:" << d->genreList << endl;
+
 	return true;
 }
 
 void EncoderLame::loadSettings(){
 	// Generate the command line arguments for the current settings
 	args.clear();
- 
+
 	Settings *settings = Settings::self();
 
 	int quality = settings->quality();
@@ -92,7 +121,7 @@ void EncoderLame::loadSettings(){
 	if (quality > 9) quality = 9;
 
 	int method = settings->bitrate_constant() ? 0 : 1 ;
-  
+
 	if (method == 0) {
 		// Constant Bitrate Encoding
 		args.append("-b");
@@ -181,10 +210,12 @@ unsigned long EncoderLame::size(long time_secs) const {
 
 long EncoderLame::readInit(long /*size*/){
 	// Create KProcess
-	d->currentEncodeProcess	= new KProcess(0,"encodeprocess");
+	d->currentEncodeProcess	= new KProcess(0);
 	QString prefix = locateLocal("tmp", "");
 	d->tempFile = new KTempFile(prefix, ".mp3");
 	d->tempFile->setAutoDelete(true);
+	d->lastErrorMessage = QString::null;
+	d->processHasExited = false;
 
 	// -x bitswap
 	// -r raw/pcm
@@ -197,17 +228,19 @@ long EncoderLame::readInit(long /*size*/){
 
 	// Read in stdin, output to the temp file
 	*d->currentEncodeProcess << "-" << d->tempFile->name().latin1();
-	
-	
+
+	//kdDebug(7117) << d->currentEncodeProcess->args() << endl;
+
+
 	connect(d->currentEncodeProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
-                         this, SLOT(receivedStdout(KProcess *, char *, int)));
+	                 this, SLOT(receivedStdout(KProcess *, char *, int)));
 	connect(d->currentEncodeProcess, SIGNAL(receivedStderr(KProcess *, char *, int)),
-                         this, SLOT(receivedStderr(KProcess *, char *, int)));
+	                 this, SLOT(receivedStderr(KProcess *, char *, int)));
 	connect(d->currentEncodeProcess, SIGNAL(wroteStdin(KProcess *)),
-                         this, SLOT(wroteStdin(KProcess *)));
+	                 this, SLOT(wroteStdin(KProcess *)));
 
 	connect(d->currentEncodeProcess, SIGNAL(processExited(KProcess *)),
-                         this, SLOT(processExited(KProcess *)));
+	                 this, SLOT(processExited(KProcess *)));
 
 	// Launch!
 	d->currentEncodeProcess->start(KProcess::NotifyOnExit, KShellProcess::All);
@@ -216,10 +249,14 @@ long EncoderLame::readInit(long /*size*/){
 
 void EncoderLame::processExited ( KProcess *process ){
 	kdDebug(7117) << "Lame Encoding process exited with: " << process->exitStatus() << endl;
+	d->processHasExited = true;
 }
 
 void EncoderLame::receivedStderr( KProcess * /*process*/, char *buffer, int /*buflen*/ ){
 	kdDebug(7117) << "Lame stderr: " << buffer << endl;
+	if ( !d->lastErrorMessage.isEmpty() )
+		d->lastErrorMessage += '\t';
+	d->lastErrorMessage += QString::fromLocal8Bit( buffer );
 }
 
 void EncoderLame::receivedStdout( KProcess * /*process*/, char *buffer, int /*length*/ ){
@@ -231,8 +268,10 @@ void EncoderLame::wroteStdin( KProcess * /*procces*/ ){
 }
 
 long EncoderLame::read(int16_t *buf, int frames){
-  if(!d->currentEncodeProcess)
+	if(!d->currentEncodeProcess)
 		return 0;
+        if (d->processHasExited)
+		return -1;
 
 	// Pipe the raw data to lame
 	char * cbuf = reinterpret_cast<char *>(buf);
@@ -281,7 +320,7 @@ long EncoderLame::readCleanup(){
 	delete d->currentEncodeProcess;
 	delete d->tempFile;
 	d->lastSize = 0;
-	
+
 	return 0;
 }
 
@@ -305,9 +344,18 @@ void EncoderLame::fillSongInfo( KCDDB::CDInfo info, int track, const QString &co
 	trackInfo.append("--tn");
 	trackInfo.append(QString("%1").arg(track+1));
 
-	trackInfo.append("--tg");
-	trackInfo.append(info.get("genre").toString());
+	const QString genre = info.get( "genre" ).toString();
+	if ( d->genreList.find( genre ) != d->genreList.end() )
+	{
+		trackInfo.append("--tg");
+		trackInfo.append(genre);
+	}
 }
 
 
+QString EncoderLame::lastErrorMessage() const
+{
+	return d->lastErrorMessage;
+}
+
 #include "encoderlame.moc"
