@@ -13,6 +13,7 @@ class KMixDockWidget;
 class MixDeviceWidget;
 class MixDevice;
 class QFrame;
+class QTime;
 
 class ViewDockAreaPopup : public ViewBase
 {
@@ -31,6 +32,7 @@ public:
     virtual void showContextMenu();
 
     QSize sizeHint() const;
+    bool justHidden();
 
 protected:
     MixDeviceWidget *_mdw;
@@ -39,9 +41,12 @@ protected:
 	 QPushButton     *_showPanelBox;
 
     void mousePressEvent(QMouseEvent *e);
+    void wheelEvent ( QWheelEvent * e );
+
 private:
     QGridLayout* _layoutMDW;
     QFrame *_frame;
+    QTime *_hideTimer;
 
 private slots:
 	 void showPanelSlot();
